package no.ntnu.ctscanarkivsystemserver.config;

import com.ulisesbocchio.jasyptspringboot.annotation.EnableEncryptableProperties;
import no.ntnu.ctscanarkivsystemserver.filter.JwtRequestFilter;
import no.ntnu.ctscanarkivsystemserver.model.database.Role;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.authentication.HttpStatusEntryPoint;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

/**
 * @author TrymV
 * @source https://www.youtube.com/watch?v=iyXne7dIn7U&list=PLqq-6Pq4lTTYTEooakHchTGglSvkZAjnE&index=4
 * @source https://www.youtube.com/watch?v=TNt3GHuayXs&t=196s
 */
@Configuration
@EnableWebSecurity
@EnableEncryptableProperties
public class SecurityConfiguration extends WebSecurityConfigurerAdapter {

    @Autowired
    UserDetailsService userService;

    @Autowired
    JwtRequestFilter jwtRequestFilter;

    /**
     * Tells Spring Boot to use my own userDetailsService class instead of the build in one.
     * @param auth
     * @throws Exception
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(userService).passwordEncoder(getPasswordEncoder());
    }

    /**
     * The Password encoder to be used.
     * @return a mew BCryptPasswordEncoder.
     */
    @Bean
    public BCryptPasswordEncoder getPasswordEncoder(){
        return new BCryptPasswordEncoder();
    }

    /**
     * Set up the role required to visit certain paths.
     * Adds the jwt filter as the first filter in the filter chain.
     * @param http
     * @throws Exception
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        http.csrf().disable().authorizeRequests()
                .antMatchers("/admin/**").hasRole(Role.ADMIN)
                .antMatchers("/academic/**").hasAnyRole(Role.ADMIN, Role.ACADEMIC)
                .antMatchers("/user/**").hasAnyRole(Role.ADMIN, Role.ACADEMIC, Role.USER)
                .antMatchers("/auth/**").permitAll()
                .anyRequest().authenticated()
                .and().addFilterBefore(jwtRequestFilter, UsernamePasswordAuthenticationFilter.class)
                .exceptionHandling().authenticationEntryPoint(new HttpStatusEntryPoint(HttpStatus.UNAUTHORIZED));
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);
        http.cors();
    }

    @Override
    @Bean
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }
}
